from ..code.base import assign, make_b, make_if
from .. import utilities as ck2
from collections import OrderedDict


class TraitSet:

    '''
    Some utility functions for dealing with groups of traits.
    '''

    def __init__(self):
        self._positive_genetic = {}
        self._agnatic = {}
        self._no_marriage = {}
        self._bad_traits = {}
        self._all_genetic = {}
        self.all_traits = {}
        self.sprites = {}
        self._localizations = {}
        # optimization for total benefit calculations
        self._total_benefits_memo = {}
        self._trait_sets = []
        self.attribute_bonuses = [
            'diplomacy', 'intrigue', 'stewardship', 'martial', 'learning', 'fertility', 'health']
        self.opinion_bonuses = ['ambition_opinion', 'vassal_opinion', 'sex_appeal_opinion',
                                'same_opinion', 'same_opinion_if_same_religion', 'opposite_opinion', 'church_opinion',
                                'castle_opinion', 'city_opinion', 'same_opinion_if_same_religion', 'liege_opinion',
                                'general_opinion', 'twin_opinion', 'dynasty_opinion', 'male_dynasty_opinion',
                                'female_dynasty_opinion', 'muslim_opinion', 'child_opinion', 'oldest_child_opinion',
                                'spouse_opinion']
        self.ai_weights = [
            'ai_rationality', 'ai_zeal', 'ai_greed', 'ai_honor', 'ai_ambition']
        # I really can't think of anything better than hardcoding.
        self._traitseries = 'genius->quick imbecile->slow'
        funcmask = 'def get_{}(self,trait): return self._get_value(trait,"{}")'
        for thing in self.attribute_bonuses + self.opinion_bonuses:
            exec(funcmask.format(thing, thing))

    def get_all_effects(self, trait):
        ret = OrderedDict()
        for bonus in self.attribute_bonuses + self.opinion_bonuses + self.ai_weights:
            value = self._get_value(trait, bonus)
            if value:
                ret[bonus] = value
        return ret

    def get_trait_block(self, trait):
        return str(self.all_traits[trait])
        return make_b(trait, str(self.all_traits[trait]))

    def best_of_set(self, list_of_traits):
        top = []
        top_v = 0
        for trait in list_of_traits:
            ben = self._total_benefit(trait)
            if ben > top_v:
                top_v = ben
                top = [trait]
            elif ben == top_v:
                top.append(trait)
        return top

    def has_best_of_set(self, set_of_traits):
        '''
        Returns a generated decision, either trait = best
        or OR = {trait = tie1 trait = tie2} if tied.
        :param set_of_traits:
        '''
        top = self.best_of_set(set_of_traits)
        if len(top) > 1:
            return make_b('OR', '\n'.join([assign('trait', x) for x in top]))
        else:
            return assign('trait', top[0])

    def worst_of_set(self, list_of_traits):
        top = []
        top_v = 1000000000
        for trait in list_of_traits:
            ben = self._total_benefit(trait)
            if ben < top_v:
                top_v = ben
                top = [trait]
            elif ben == top_v:
                top.append(trait)
        return top

    def has_worst_of_set(self, set_of_traits):
        '''
        Returns a generated decision, either trait = best
        or OR = {trait = tie1 trait = tie2} if tied.
        :param set_of_traits:
        '''
        top = self.worst_of_set(set_of_traits)
        if len(top) > 1:
            return make_b('OR', '\n'.join([assign('trait', x) for x in top]))
        else:
            return assign('trait', top[0])

    def perfect_genes(self):
        ret = ''
        for set in self.get_pos_gen_traitsets():
            if len(set) > 1:
                ret += self.has_best_of_set(set)
            else:
                ret += assign('trait', list(set)[0])
        return ret

    def all_good_genes(self):
        ret = ''
        for set in self.get_pos_gen_traitsets():
            if len(set) > 1:
                ret += make_b('OR',
                              '\n'.join([assign('trait', x) for x in set]))
            else:
                ret += assign('trait', list(set)[0])
        return ret

    def _get_traitsets(self, container=None, filter=None):
        potential = []
        for trait in container:
            if not filter or filter(trait):
                this_set = set()
                this_set.add(trait)
                opp = self.get_opposites(trait)
                # print trait
                # print opp
                if opp:
                    for other in opp:
                        if not filter or filter(other):
                            this_set.add(other)
                if not filter or filter(trait):
                    potential.append(this_set)
        master = []
        for thing in potential:
            if master.count(thing) == 0:
                master.append(thing)
        return master

    def get_traitsets(self, filter=None):
        return self._get_traitsets(self.all_traits, filter)

    def get_genetic_traitsets(self, filter=None):
        '''
        builds 'sets' of traits that are exclusive.
        :param filter: Optional function to filter
            results.
        '''
        return self._get_traitsets(self._all_genetic, filter)

    def is_epidemic(self, trait):
        return self._get_value(trait, 'is_epidemic')

    def is_health(self, trait):
        health = self._get_value(trait, 'is_health')
        illness = self._get_value(trait, 'is_illness')
        epidemic = self._get_value(trait, 'is_epidemic')
        return health and not (epidemic or illness)

    def is_illness(self, trait):
        illness = self._get_value(trait, 'is_illness')
        epidemic = self._get_value(trait, 'is_epidemic')
        return illness and not epidemic

    def get_health_traitsets(self):
        return self.get_traitsets(self.is_health)

    def get_illness_traitsets(self):
        return self.get_traitsets(self.is_illness)

    def get_pos_gen_traitsets(self):
        return self.get_genetic_traitsets(self.is_positive)

    def get_neg_gen_traitsets(self):
        return self.get_genetic_traitsets(self.is_negative)

    def _trait_cmp(self, trait1, trait2):
        all_stronger = True
        all_weaker = True
        both_pos = self.is_positive(trait1) and self.is_positive(trait2)
        both_neg = self.is_negative(trait1) and self.is_negative(trait2)
        if not (both_neg or both_pos):
            # no valid comparison
            return False, False
        for bonus in self.attribute_bonuses:
            t1v = self._get_value(trait1, bonus)
            t2v = self._get_value(trait2, bonus)
            if t1v:
                if not t2v:
                    all_stronger = False
                    all_weaker = False
                else:
                    if both_pos:
                        if t1v < t2v:
                            all_stronger = False
                        else:
                            all_weaker = False
                    if both_neg:
                        if t1v < t2v:
                            all_stronger = False
                        else:
                            all_weaker = False
        print trait1, trait2, both_pos, both_neg, all_stronger, all_weaker
        return all_stronger, all_weaker

    def is_stronger_version(self, trait1, trait2):
        '''
        is trait 1 a stronger version of trait 2.
        :param trait1: trait 1 name
        :param trait2: trait 2 name
        '''
        if '{}->{}'.format(trait1, trait2) in self._traitseries:
            return True
        return False
        a, b = self._trait_cmp(trait1, trait2)
        return a

    def is_weaker_version(self, trait1, trait2):
        '''
        is trait 1 a lesser version of trait 2.
        :param trait1: trait 1 name
        :param trait2: trait 2 name
        '''
        if '{}->{}'.format(trait2, trait1) in self._traitseries:
            return True
        return False
        a, b = self._trait_cmp(trait1, trait2)
        return b

    def is_equivalent(self, trait1, trait2):
        return self._total_benefit(trait1) == self._total_benefit(trait2)

    def is_positive(self, trait):
        return self._total_benefit(trait) > 0

    def is_negative(self, trait):
        return self._total_benefit(trait) < 0

    def _total_benefit(self, trait):
        if trait in self._total_benefits_memo:
            return self._total_benefits_memo[trait]
        total = 0
        for thing in self.attribute_bonuses:
            val = self._get_value(trait, thing)
            if val:
                total += int(float(val) * 100)
        self._total_benefits_memo[trait] = total
        return total

    def get_name_localization(self, trait):
        if trait in self._localizations:
            return self._localizations[trait]
        return None

    def get_desc_localization(self, trait):
        if trait + '_desc' in self._localizations:
            return self._localizations[trait + '_desc']
        return None

    def localize_trait(self, trait, lines):
        ret = []
        for line, loca in zip(lines, self._localizations[trait]):
            if loca == '':
                loca = trait
            ret.append(line.format(loca))
        return ret

    def add_localizations(self, localization_dict):
        for trait in self.all_traits:
            if trait in localization_dict:
                self._localizations[trait] = localization_dict[trait]
            if trait + '_desc' in localization_dict:
                self._localizations[
                    trait + '_desc'] = localization_dict[trait + '_desc']

    def _get_value(self, trait, variable):
        if trait not in self.all_traits:
            print trait, 'not found...'
            return None
        for thing in self.all_traits[trait]:
            if hasattr(thing, 'value'):
                if thing.variable == variable:
                    return thing.value
        return None

    def get_inherit_chance(self, trait):
        return self._get_value(trait, 'inherit_chance')
        for thing in self.all_traits[trait]:
            if hasattr(thing, 'value'):
                if thing.variable == 'inherit_chance':
                    return thing.value
        return None

    def get_opposites(self, trait):
        for thing in self.all_traits[trait]:
            if hasattr(thing, '__name__') and thing.__name__ == 'opposites':
                return [str(x).partition('#')[0].replace('\n', '')
                        for x in thing]
        return []

    def num_good(self):
        return len(self._positive_genetic.keys())

    def is_genetic(self, trait):
        return trait in self._all_genetic.keys()

    def add(self, trait):
        # for my purposes, I can only add the 'second' version,
        #   so lets treat it as replacement
        if trait.__name__ in self.all_traits:
            # clear previous values, if they exist
            self._all_genetic.pop(trait.__name__, 'None')
            self._positive_genetic.pop(trait.__name__, 'None')
            self._bad_traits.pop(trait.__name__, 'None')
            self._agnatic.pop(trait.__name__, 'None')
            self._no_marriage.pop(trait.__name__, 'None')
        self.all_traits[trait.__name__] = trait
        inherit = self.get_inherit_chance(trait.__name__)
        # if trait.__name__ == 'saoshyant_descendant':
        #   print str(trait)
        if inherit and int(inherit) < 100 and int(inherit) > 0:
            # its not really worth considering traits with 100% chance to be inherited
            #    and mod authors sometimes redundantly specifiy 0 chance.
            if trait.__name__ != 'syphilitic':
                # as an aside, instead of a hardcoded exception
                #    perhaps look at things that are
                #    is_illness or is_health instead...
                # this is a disease, and can be passed on
                #    from a parent to a child, but isn't
                #    genetic, as opposed to dwarfism.
                self._all_genetic[trait.__name__] = trait
            val = self._total_benefit(trait.__name__)
            # if '-' not in str(trait):
            if val > 0:
                # print 'positive trait',trait.__name__,val
                # print trait.__name__, self._total_benefit(trait.__name__)
                self._positive_genetic[trait.__name__] = trait
            else:
                self._bad_traits[trait.__name__] = trait
        if 'agnatic' in str(trait):
            self._agnatic[trait.__name__] = trait
            # else:
            # self._bad_traits[trait.__name__]=trait
        if 'cannot_marry' in str(trait):
            self._no_marriage[trait.__name__] = trait
        if 'inbred' in str(trait):
            # inbred is always bad
            self._bad_traits[trait.__name__] = trait

    def report(self):
        print 'positive genetic', self._positive_genetic.keys()
       # print 'positive agnatic', self._positive_agnatic.keys()
        print 'no_marriage', self._no_marriage.keys()
        print 'bad', self.bad_traits.keys()

    def good_genes(self):
        return self.or_traitlist(self._positive_genetic)

    def or_traitlist(self, list_of_traits):
        ret = ''
        for trait in list_of_traits:
            ret += assign('trait', trait)
        if len(list_of_traits) > 1:
            return make_b('OR', ret)
        return ret

    def not_traitlist(self, list_of_traits):
        ret = ''
        for trait in list_of_traits:
            ret += assign('trait', trait)
        if len(list_of_traits) >= 1:
            return make_b('NOT', ret)
        return ret

    def and_traitlist(self, list_of_traits):
        ret = ''
        for trait in list_of_traits:
            ret += assign('trait', trait)
        if len(list_of_traits) >= 1:
            return make_b('AND', ret)
        return ret

    def bloodline(self):
        # practically speaking, not terribly useful
        return self.or_traitlist(self._agnatic)

    def _bad_trait_string(self):
        ret = ''
        for trait in self._bad_traits:
            ret += assign('trait', trait)
        return ret

    def bad_traits(self):
        return self.or_traitlist(self._positive_genetic)
        return make_b('OR', self._bad_trait_string())

    def no_bad_traits(self):
        return make_b('NOT', self._bad_trait_string())

    def _marriage_blocking_traits_string(self):
        ret = ''
        for trait in self._no_marriage:
            ret += assign('trait', trait)
        return ret

    def add_sprites(self, interfaceparse):
        for thing in interfaceparse:
            if thing.__name__ == 'spriteTypes':
                for sprite in thing:
                    if hasattr(sprite, '__name__') and sprite.__name__ == 'spriteType':
                        if 'GFX_trait' in str(sprite):
                            traitname = str(sprite).partition(
                                'GFX_trait_')[2].partition('"')[0]
                            # print 'found ', traitname
                            self.sprites[traitname] = sprite

    def load_path(self, path):
        raw = ck2.get_mod_traits(path)
        for trait in raw:
            self.add(trait)
        self.add_localizations(ck2.get_mod_localization(path))
        self.add_sprites(ck2.get_mod_interface(path))

    def add_all(self, iterable):
        for trait in iterable:
            self.add(trait)

    def update(self, other_traitset):
        self.update_traits(other_traitset)
        self.update_loca(other_traitset)
        self.update_sprites(other_traitset)

    def update_traits(self, other_traitset):
        self._positive_genetic.update(other_traitset._positive_genetic)
        self._agnatic.update(other_traitset._agnatic)
        self._no_marriage.update(other_traitset._no_marriage)
        self._bad_traits.update(other_traitset._bad_traits)
        self._all_genetic.update(other_traitset._all_genetic)
        self.all_traits.update(other_traitset.all_traits)

    def update_sprites(self, other_traitset):
        self.sprites.update(other_traitset.sprites)

    def update_loca(self, other_traitset):
        self._localizations.update(other_traitset._localizations)

    def not_marriable(self):
        return make_b('OR', self._marriage_blocking_traits_string())

    def marriable(self):
        return make_b('NOT', self._marriage_blocking_traits_string())
